/**
 * Copyright (c) 2021 Dominic Msters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "input.h"

input_t * inputInit(platform_t *platform, uint32_t inputBindCount) {
  // Create the input manager
  input_t *input = malloc(sizeof(input_t));
  if(!input) return NULL;

  // Setup the binds
  input->inputBindCount = inputBindCount;
  input->binds = malloc(sizeof(inputbind_t) * inputBindCount);
  if(input->binds == NULL) {
    free(input);
    return NULL;
  }

  // Create the current input states
  input->current = malloc(sizeof(float) * inputBindCount);
  if(input->current == NULL) {
    free(input->binds);
    free(input);
    return NULL;
  }

  // Create the previous input states.
  input->previous = malloc(sizeof(float) * inputBindCount);
  if(input->previous == NULL) {
    free(input->current);
    free(input->binds);
    free(input);
    return NULL;
  }

  // Create the input actuate times array.
  input->times = malloc(sizeof(float) * inputBindCount);
  if(input->times == NULL) {
    free(input->previous);
    free(input->current);
    free(input->binds);
    free(input);
    return NULL;
  }

  // Pass off the input manager
  return input;
}

void inputUpdate(platform_t *platform, input_t *input) {
  // Flip the states to save memory.
  float * currentCurrent = input->current;
  input->current = input->previous;
  input->previous = currentCurrent;

  // Now look at each bind, get the state from the platform
  for(uint32_t i = 0; i < input->inputBindCount; i++) {
    // For this bind, get the sources.
  }
}

void inputDispose(input_t *input) {
  free(input->times);
  free(input->previous);
  free(input->current);
  free(input->binds);
  free(input);
}

inputbindmap_t inputBind(input_t *input, inputbind_t bind,
  platforminputsource_t *source
) {
  return NULL;
}

bool inputIsDown(input_t *input, inputbind_t binding) {
  return input->current[binding] != 0;
}

bool inputIsUp(input_t *input, inputbind_t binding) {
  return input->current[binding] == 0;
}

bool inputIsPressed(input_t *input, inputbind_t binding) {
  return (
    input->previous[binding] == 0 &&
    input->current[binding] != 0
  );
}

bool inputIsReleased(input_t *input, inputbind_t binding) {
  return (
    input->current[binding] == 0 &&
    input->previous[binding] != 0
  );
}

float inputGetAxis(input_t *input, inputbind_t binding) {
  return input->current[binding];
}

float inputGetFullAxis(input_t *input, inputbind_t positive,
  inputbind_t negative
) {
  return input->current[negative] + input->current[positive];
}

float inputGetAccuated(input_t *input, inputbind_t binding) {
  return input->times[binding];
}